package org.recursed.w.client.about;

import org.recursed.w.client.characteristics.HasVisibility;
import org.recursed.w.client.place.AboutPlace;
import org.recursed.w.client.place.PlaceChangeEvent;
import org.recursed.w.client.place.PlaceService;
import org.recursed.w.client.place.SummaryPlace;
import org.recursed.w.client.place.PlaceChangeEvent.PlaceChangeHandler;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
public class AboutPresenter {

    public interface Display {
        public HasClickHandlers getCloseButton();

        public HasVisibility getVisibility();
    }

    private Display display;
    private PlaceService placeService;

    @Inject
    public AboutPresenter(Display display, PlaceService placeService) {
        this.display = display;
        this.placeService = placeService;
        initButtons();
        initPlaceService();
    }

    private void initPlaceService() {
        placeService.addPlaceChangeHandler(new PlaceChangeHandler() {

            @Override
            public void onPlaceChangeEvent(PlaceChangeEvent pce) {
                if (pce.getPlace() instanceof AboutPlace) {
                    display.getVisibility().setVisibility(true);
                }
            }
        });
    }

    private void initButtons() {
        display.getCloseButton().addClickHandler(new ClickHandler() {

            @Override
            public void onClick(ClickEvent event) {
                placeService.changePlace(new SummaryPlace());
                display.getVisibility().setVisibility(false);
            }
        });
    }
}
