package org.recursed.w.client.headlines;

import java.util.ArrayList;

import org.recursed.w.client.characteristics.HasHeadline;
import org.recursed.w.client.place.PlaceChangeEvent;
import org.recursed.w.client.place.PlaceService;
import org.recursed.w.client.place.WordDetailPlace;
import org.recursed.w.client.place.PlaceChangeEvent.PlaceChangeHandler;
import org.recursed.w.client.rpc.RPCService;
import org.recursed.w.shared.headlines.GetHeadlines;
import org.recursed.w.shared.headlines.GetHeadlinesCallback;
import org.recursed.w.shared.headlines.HeadlineDTO;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Window;
import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
public class HeadlinesPresenter  {

    private RPCService rpcService;
    private GetHeadlinesCallback callback;
    private ArrayList<HeadlineDTO> currentHeadlines;
    private int counter;
    private HasHeadline[] hasHeadlines;
    
    public interface Display {
        public static final int MAX_HEADLINES = 15;
        public HasHeadline getHeadline(int position);
    }

    @Inject
    public HeadlinesPresenter(RPCService rpcService, PlaceService placeService, Display display) {
        this.rpcService = rpcService;
        placeService.addPlaceChangeHandler(new PlaceChangeHandler() {
            
            @Override
            public void onPlaceChangeEvent(PlaceChangeEvent pce) {
                if (pce.getPlace() instanceof WordDetailPlace) {
                    WordDetailPlace wordDetailPlace = (WordDetailPlace) pce.getPlace();
                    int wordId = wordDetailPlace.getWordId();
                    populateHeadlinesForWord(wordId);
                }
            }
        });
        callback = new GetHeadlinesCallback() {
            
            @Override
            public void gotHeadlines(ArrayList<HeadlineDTO> headlines) {
                currentHeadlines = headlines;
                counter = 0;
                repopulate();
            }
        };
        hasHeadlines = new HasHeadline[Display.MAX_HEADLINES];
        for (int x=0; x<Display.MAX_HEADLINES; x++) {
            final int position = x;
            hasHeadlines[x] = display.getHeadline(x);
            hasHeadlines[x].getClickHandlers().addClickHandler(new ClickHandler() {
                
                @Override
                public void onClick(ClickEvent event) {
                    showLink(position);
                }
            });
        }
    }

    private void populateHeadlinesForWord(int wordId) {
        rpcService.execute(new GetHeadlines(wordId), callback);
    }

    private void repopulate() {
        for (int x=0; x<Display.MAX_HEADLINES; x++) {
            hasHeadlines[x].getString().setText(null);
        }
        for (int x=0; x<Display.MAX_HEADLINES && x+counter < currentHeadlines.size(); x++) {
            HeadlineDTO dto = currentHeadlines.get(x+counter);
            hasHeadlines[x].getString().setText(dto.getTitle());
        }
    }

    public void showLink(int position) {
        HeadlineDTO toShow = currentHeadlines.get(position + counter);
        String link = toShow.getLink();
        Window.open(link, "_blank", null);
    }
}
