package org.recursed.w.client.place;

import com.google.gwt.user.client.Command;

public abstract class Place  {

    private String urlShortCode;
    /**
     * Mandatory..
     * @param urlShortCode
     */
    public Place(String urlShortCode) {
        this.urlShortCode = urlShortCode;
    }

    /**
     * @return A representation for this place's name on the url.
     * This should be something like 's' for 'SomePlace'
     */
    public final String getURLShortCode() {
        return urlShortCode;
    }

    /**
     * @return Optional arguments for the place based on the DTOs
     * within it. Eg: SomePlace may contain a Person with id 1, so
     * it would return something like {"p", "1"}
     */
    public abstract String[] getTokenizedArguments();

    /**
     * This method is fired on a brower driven place change (or
     * when a url is changed). The supplied tokens are what this place
     * would have returned via the tokenizeArguments method. This
     * object should take the opportunity to flesh out any models/dtos
     * that it may normally provide. When done, it should fire the
     * supplied command.
     * 
     * @param tokens
     * @param c
     */
    public void inflateAndFire(Command c) {
        c.execute();
    }

    public abstract void setTokens(String[] tokens);

}
