package org.recursed.w.client.place;

import java.util.HashMap;
import java.util.Map;

import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
public class PlaceFactory implements PlaceService.Factory {

    private Map<String, Class<? extends Place>> places;

    @Inject
    private PlaceFactory() {
        places = new HashMap<String, Class<? extends Place>>();
        places.put(SummaryPlace.SHORT_CODE, SummaryPlace.class);
        places.put(WordDetailPlace.CODE, WordDetailPlace.class);
        places.put(AboutPlace.SHORT_CODE, AboutPlace.class);
    }

    @Override
    public Place getDefaultPlace() {
        return new SummaryPlace();
    }

    @Override
    public Place getPlace(String urlShortCode, String[] tokens) {
        Place p = null;
        if (urlShortCode.equals("s")) {
            p = new SummaryPlace();
        } else if (urlShortCode.equals("w")) {
            p = new WordDetailPlace();
        } else if (urlShortCode.equals("a")) {
            p = new AboutPlace();
        }
        if (p == null) {
            p = getDefaultPlace();
        }
        p.setTokens(tokens);
        return p;
    }
}