package org.recursed.w.client.relationshipgraph;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import org.recursed.w.client.characteristics.HasChartControls;
import org.recursed.w.client.place.Place;
import org.recursed.w.client.place.PlaceChangeEvent;
import org.recursed.w.client.place.PlaceService;
import org.recursed.w.client.place.WordDetailPlace;
import org.recursed.w.client.place.PlaceChangeEvent.PlaceChangeHandler;
import org.recursed.w.client.rpc.RPCService;
import org.recursed.w.client.worddetail.GetWordDetailCallback;
import org.recursed.w.shared.GetGroupedWordDetailAction;
import org.recursed.w.shared.GetWordDetailAction;
import org.recursed.w.shared.GroupedWordDetail;
import org.recursed.w.shared.WordDTO;
import org.recursed.w.shared.WordDetail;

import com.google.inject.Inject;

public class RelationshipGraphPresenter implements PlaceChangeHandler {

    private WordDTO core;

    public interface Display {
        public HasChartControls getControls();

        public void setCoreWord(WordDTO wordDTO);

        public void addLinkage(WordDTO from, WordDTO to);
        
        public void addSecondaryLinkage(WordDTO from, WordDTO to);

        public void renderSecondaryLinkages();
    }

    private Display display;
    private RPCService rpcService;
    private PlaceService placeService;
    private int expectingDataFor; 

    @Inject
    public RelationshipGraphPresenter(Display display,
            RPCService rpcService, PlaceService placeService) {
        this.display = display;
        this.rpcService = rpcService;
        this.placeService = placeService;
        this.placeService.addPlaceChangeHandler(this);
    }

    private void reset(int id) {
        GetWordDetailAction gwa = new GetWordDetailAction(id, GetWordDetailAction.DEFAULT_MAX_WORDS);
        rpcService.execute(gwa, new GetWordDetailCallback() {
            
            @Override
            public void got(WordDetail response) {
                WordDTO coreWord = response.getCoreWord();
                int gotId = coreWord.getId();
                if (gotId != expectingDataFor) {
                    return;
                }
                resetGraph(response);
            }
        });
    }

    private void resetGraph(WordDetail result) {
        core = result.getCoreWord();
        display.getControls().clear();
        display.setCoreWord(core);
        HashSet<WordDTO> relatedWords = result.getRandomRelatedWords(6);
        GetGroupedWordDetailAction moreWords = new GetGroupedWordDetailAction();
        for (WordDTO relatedWord : relatedWords) {
            display.addLinkage(core, relatedWord);
            moreWords.addWord(relatedWord);
        }
        rpcService.execute(moreWords, new GetGroupedWordDetailCallback() {

            @Override
            public void got(GroupedWordDetail result) {
                HashMap<WordDTO,ArrayList<WordDTO>> linkages = result.getLinkages();
                Set<WordDTO> froms = linkages.keySet();
                for (WordDTO from : froms) {
                    ArrayList<WordDTO> tos = linkages.get(from);
                    for (WordDTO to : tos) {
                        display.addSecondaryLinkage(from, to);
                        //GWT.log("Adding " + from.getLabel() + " to " + to.getLabel(), null);
                    }
                }
                display.renderSecondaryLinkages();
            }
        });
    }

    @Override
    public void onPlaceChangeEvent(PlaceChangeEvent pce) {
        Place place = pce.getPlace();
        if (!(place instanceof WordDetailPlace)) {
            return;
        }
        WordDetailPlace wdp = (WordDetailPlace) place;
        int id = wdp.getWordId();
        expectingDataFor = id;
        reset(id);
    }

}
