package org.recursed.w.client.rpc;

import org.recursed.w.shared.corerpc.Action;
import org.recursed.w.shared.corerpc.BatchedAction;
import org.recursed.w.shared.corerpc.Response;

import com.google.gwt.user.client.Timer;
import com.google.inject.Singleton;

@Singleton
public class BatchingRPCService implements RPCService {

    private RPCService impl;
    private Timer t;
    private BatchedAction currentBatch;
    private BatchedActionCallback currentCallback;
    private int counter;
    private int batchCounter;
    private boolean freshBatch;

    public BatchingRPCService(RPCService impl) {
        this.impl = impl;
        t = new Timer() {

            @Override
            public void run() {
                fireBatch();
            }
        };
        freshBatch = true;
    }

    @Override
    public <A extends Action, R extends Response> void execute(A action,
            ActionCallback<R> callback) {
        //GWT.log("Batch " + batchCounter + " -> " + action.getClass(), null);
        if (freshBatch) {
            currentBatch = new BatchedAction();
            currentCallback = new BatchedActionCallback();
            counter = 0;
            t.schedule(30);
            freshBatch = false;
        }
        int actionID = currentBatch.addAction(counter, action);
        currentCallback.addCallback(actionID, callback);
        counter++;
    }

    private void fireBatch() {
        impl.execute(currentBatch, currentCallback);
        freshBatch = true;
        batchCounter ++;
    }

}
