package org.recursed.w.client.search;

import com.google.gwt.event.dom.client.FocusEvent;
import com.google.gwt.event.dom.client.FocusHandler;
import com.google.gwt.event.logical.shared.HasSelectionHandlers;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.SuggestBox;
import com.google.gwt.user.client.ui.SuggestOracle.Suggestion;
import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
public class SearchDisplay implements SearchPresenter.Display {
    private static final String SEARCH_CAPSULE_DIV = "search_capsule";
    private static final String STYLE_SUGGEST_BOX = "search_input";
    private static final String STYLE_POPUP = "search_popup";

    private SuggestBox wordSearchSuggestBox;

    @Inject
    public SearchDisplay(WordSuggestOracle suggestOracle) {
        RootPanel searchCapsuleDiv = RootPanel.get(SEARCH_CAPSULE_DIV);
        wordSearchSuggestBox = new SuggestBox(suggestOracle);
        wordSearchSuggestBox.setText("click here to search");
        wordSearchSuggestBox.setStyleName(STYLE_SUGGEST_BOX);
        wordSearchSuggestBox.setPopupStyleName(STYLE_POPUP);
        wordSearchSuggestBox.getTextBox().addFocusHandler(new FocusHandler() {

            @Override
            public void onFocus(FocusEvent event) {
                wordSearchSuggestBox.setText("");
            }
        });
        searchCapsuleDiv.add(wordSearchSuggestBox);
    }

    public HasSelectionHandlers<Suggestion> getSelectionHandlers() {
         return wordSearchSuggestBox;
    }

    @Override
    public int getTabIndex() {
        return wordSearchSuggestBox.getTabIndex();
    }

    @Override
    public void setAccessKey(char key) {
        wordSearchSuggestBox.setAccessKey(key);
    }

    @Override
    public void setFocus(boolean focused) {
        wordSearchSuggestBox.setFocus(focused);
    }

    @Override
    public void setTabIndex(int index) {
        wordSearchSuggestBox.setTabIndex(index);
    }

}
