package org.recursed.w.client.search;

import java.util.ArrayList;

import org.recursed.w.client.rpc.ActionCallback;
import org.recursed.w.client.rpc.RPCService;
import org.recursed.w.shared.WordDTO;
import org.recursed.w.shared.search.GetSuggestedWordsAction;
import org.recursed.w.shared.search.SuggestedWordsResponse;

import com.google.gwt.user.client.ui.SuggestOracle;
import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
public class WordSuggestOracle extends SuggestOracle {

    private RPCService rpcService;

    @Inject
    public WordSuggestOracle (RPCService rpcService) {
        this.rpcService = rpcService;
    }
    
    @Override
    public void requestSuggestions(Request request, Callback suggestCallback) {
        GetSuggestedWordsAction action = new GetSuggestedWordsAction(request.getQuery());
        SuggestionRPCCallback callback = new SuggestionRPCCallback(request, suggestCallback);
        rpcService.execute(action, callback);
    }

    private class SuggestionRPCCallback extends ActionCallback<SuggestedWordsResponse> {

        private Request uiRequest;
        private Callback uiCallback;

        SuggestionRPCCallback(Request uiRequest, Callback uiCallback) {
            this.uiRequest = uiRequest;
            this.uiCallback = uiCallback;
        }

        @Override
        public void got(SuggestedWordsResponse response) {
            ArrayList<WordDTO> words = response.getWords();
            Response uiResponse = new Response();
            uiResponse.setSuggestions(words);
            uiCallback.onSuggestionsReady(uiRequest, uiResponse);
        }
    }


}
