package org.recursed.w.client.summary;

import java.util.HashMap;
import java.util.Map;

import org.recursed.w.client.characteristics.HasWord;
import org.recursed.w.shared.ChronoBands;

import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.HTMLPanel;
import com.google.gwt.user.client.ui.HasText;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
public class SummaryDisplay implements SummaryPresenter.Display {

    private static final String TRENDS = "trends";
    private static final String TREND_WORD = "trend_word_label";
    private static final String TRENDS_UL_DIV = "trends_div";
    private HTMLPanel mainArea;
    private Map<String, HasWord> hasWords;

    @Inject
    private SummaryDisplay() {
        final RootPanel trendsDiv = RootPanel.get(TRENDS);
        mainArea = new HTMLPanel(TRENDS_LIST);
        mainArea.setStyleName(TRENDS_UL_DIV);
        trendsDiv.add(mainArea);
        mainArea.sinkEvents(Event.FOCUSEVENTS | Event.ONCLICK | Event.ONBLUR);
        hasWords = new HashMap<String, HasWord>();
        generateHasWords();
    }

    private void generateHasWords() {
        for (ChronoBands band : ChronoBands.values()) {
            for (int x = 0; x < 5; x++) {
                String elementID = band.getIDPrefix() + "_" + (x + 1);
                final Label l = new Label();
                mainArea.addAndReplaceElement(l, elementID);
                l.setStylePrimaryName(TREND_WORD);
                HasWord hw = new HasWord() {
                    @Override
                    public HasText getText() {
                        return l;
                    }
                    
                    @Override
                    public HasClickHandlers getClickHandlers() {
                        return l;
                    }
                };
                hasWords.put(elementID, hw);
            }
        }
    }

    @Override
    public HasWord getWord(ChronoBands band, int position) {
        String elementID = band.getIDPrefix() + "_" + (position + 1);
        return hasWords.get(elementID);
    }

    
    private static final String TRENDS_LIST = 
"     <ul class=\"trends\"> \n"+
"        <li class=\"chrono_label\">this hour:</li> \n"+
"            <li class=\"trend_word\"><a id=\"h_1\"></a></li>\n"+
"            <li class=\"trend_word\"><a id=\"h_2\"></a></li>\n"+
"            <li class=\"trend_word\"><a id=\"h_3\"></a></li>\n"+
"            <li class=\"trend_word\"><a id=\"h_4\"></a></li>\n"+
"            <li class=\"trend_word\"><a id=\"h_5\"></a></li>\n"+
"        <li class=\"chrono_break\"></li>\n"+
"        <li class=\"chrono_label\">today:</li>\n"+
"            <li class=\"trend_word\"><a id=\"d_1\">...</a></li>\n"+
"            <li class=\"trend_word\"><a id=\"d_2\"></a></li>\n"+
"            <li class=\"trend_word\"><a id=\"d_3\"></a></li>\n"+
"            <li class=\"trend_word\"><a id=\"d_4\"></a></li>\n"+
"            <li class=\"trend_word\"><a id=\"d_5\"></a></li>\n"+
"        <li class=\"chrono_break\"></li>\n"+
"        <li class=\"chrono_label\">this week:</li>\n"+
"            <li class=\"trend_word\"><a id=\"w_1\">...</a></li>\n"+
"            <li class=\"trend_word\"><a id=\"w_2\"></a></li>\n"+
"            <li class=\"trend_word\"><a id=\"w_3\"></a></li>\n"+
"            <li class=\"trend_word\"><a id=\"w_4\"></a></li>\n"+
"            <li class=\"trend_word\"><a id=\"w_5\"></a></li>\n"+
"      </ul>";
}
