package org.recursed.w.client.summary;

import org.recursed.w.client.characteristics.HasWord;
import org.recursed.w.client.place.Place;
import org.recursed.w.client.place.PlaceChangeEvent;
import org.recursed.w.client.place.PlaceService;
import org.recursed.w.client.place.SummaryPlace;
import org.recursed.w.client.place.WordDetailPlace;
import org.recursed.w.client.place.PlaceChangeEvent.PlaceChangeHandler;
import org.recursed.w.client.relationshipgraph.RelationshipGraphPresenter;
import org.recursed.w.client.rpc.RPCService;
import org.recursed.w.shared.ChronoBands;
import org.recursed.w.shared.GetTopWordsInChronoBands;
import org.recursed.w.shared.TopWordsInChronoBands;
import org.recursed.w.shared.WordDTO;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.inject.Inject;

public class SummaryPresenter implements PlaceChangeHandler {

    private static final int PRESENT_WORDS_FOR = 1000 * 10;
    public interface Display {
        public HasWord getWord(ChronoBands band, int position);
    }

    private Display display;
    private RPCService rpcService;
    private PlaceService placeService;
    private TopWordsInChronoBands currentTopWords;
    private int currentMarker;
    private Timer timedPresenter;
    private static boolean justBooted = true;
    private boolean pickAWord = false;

    @Inject
    private SummaryPresenter(Display display, RPCService rpcService,
            PlaceService placeService,
            RelationshipGraphPresenter relationshipGraphPresenter) {
        placeService.addPlaceChangeHandler(this);
        this.display = display;
        this.placeService = placeService;
        this.rpcService = rpcService;
        Timer t = new Timer() {

            @Override
            public void run() {
                getData();
            }

        };
        // get data every 10 minutes
        t.scheduleRepeating(1000 * 60 * 10);
        getData();
        for (final ChronoBands band : ChronoBands.values()) {
            for (int x = 0; x < 5; x++) {
                HasWord hw = display.getWord(band, x);
                final int position = x;
                hw.getClickHandlers().addClickHandler(new ClickHandler() {
                    @Override
                    public void onClick(ClickEvent event) {
                        firePlaceChangeFor(band, position+(currentMarker-5));
                    }
                });
            }
        }
        timedPresenter = new Timer() {
            @Override
            public void run() {
                populate();
            }
        };
    }

    private void getData() {
        GetTopWordsInChronoBands get = new GetTopWordsInChronoBands();
        rpcService.execute(get, new GetTopWordsInChronoBandsCallback() {

            @Override
            public void got(TopWordsInChronoBands result) {
                currentTopWords = result;
                currentMarker = 0;
                resetTimerPresenter();
            }

            @Override
            public void onFailure(Throwable caught) {
                RootPanel.get().add(new Label("rpc failed"));
            }
        });
    }

    private void resetTimerPresenter() {
        populate();
        timedPresenter.scheduleRepeating(PRESENT_WORDS_FOR);
    }

    private void populate() {
        ChronoBands[] bands = ChronoBands.values();
        for (final ChronoBands band : bands) {
            final WordDTO[] words = currentTopWords.getWords(band);
            for (int x = 0; x<5 && x<words.length; x++) {
                HasWord hw = display.getWord(band, x);
                hw.getText().setText(words[currentMarker+x].getDisplayString());
                if (pickAWord) {
                    placeService.changePlace(new WordDetailPlace(words[currentMarker+x]));
                    pickAWord = false;
                }
            }
        }
        currentMarker += 5;
        if (currentMarker == 30) {
            currentMarker = 0;
        }
    }

    private void firePlaceChangeFor(ChronoBands band, int position) {
        WordDTO dto = currentTopWords.getWords(band)[position];
        WordDetailPlace w = new WordDetailPlace(dto);
        placeService.changePlace(w);
    }

    @Override
    public void onPlaceChangeEvent(PlaceChangeEvent pce) {
        Place p = pce.getPlace();
        if (p instanceof SummaryPlace) {
            if (justBooted) {
                pickAWord = true;
            }
        } else {
            // display.getVisibility().setVisibility(false);
        }
        justBooted = false;
    }

}
