package org.recursed.w.client.worddetail;

import java.util.ArrayList;

import org.recursed.w.client.characteristics.HasWord;
import org.recursed.w.client.place.PlaceChangeEvent;
import org.recursed.w.client.place.PlaceService;
import org.recursed.w.client.place.WordDetailPlace;
import org.recursed.w.client.place.PlaceChangeEvent.PlaceChangeHandler;
import org.recursed.w.client.rpc.RPCService;
import org.recursed.w.shared.GetWordDetailAction;
import org.recursed.w.shared.WordDTO;
import org.recursed.w.shared.WordDetail;
import org.recursed.w.shared.trends.TrendDTO;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
public class ChosenWordPresenter implements PlaceChangeHandler {

    public interface Display {

        public void reset();
        public HasTrendArea getTrendArea();
    }


    public interface HasTrendArea {

        public HasWord getNextWord();
        public void setDateRange(String string);
    }
    
    private Display display;
    private RPCService rpcService;
    private PlaceService placeService;
    private ArrayList<HandlerRegistration> cleanup = new ArrayList<HandlerRegistration>();
    private int currentlyFetching;

    @Inject
    private ChosenWordPresenter(PlaceService placeService,
            RPCService rpcService, Display display) {
        placeService.addPlaceChangeHandler(this);
        this.rpcService = rpcService;
        this.display = display;
        this.placeService = placeService;
    }

    private void cleanupRegistration() {
        for (HandlerRegistration h : cleanup) {
            h.removeHandler();
        }
        cleanup = new ArrayList<HandlerRegistration>();
    }

    private void populateWord(int id) {
        this.currentlyFetching = id;
        GetWordDetailAction gwa = new GetWordDetailAction(id, GetWordDetailAction.DEFAULT_MAX_WORDS);
        rpcService.execute(gwa, new GetWordDetailCallback() {
            
            @Override
            public void got(WordDetail response) {
                WordDTO coreWord = response.getCoreWord();
                if (coreWord.getId() != currentlyFetching) {
                    return;
                }
                display.reset();
                ArrayList<TrendDTO> trends = response.getTrends();
                for (TrendDTO trend : trends) {
                    ArrayList<String> dates = trend.getDates();
                    ArrayList<WordDTO> words = trend.getWords();
                    String startDate = dates.get(0);
                    String endDate = dates.get(dates.size()-1);
                    HasTrendArea trendArea = display.getTrendArea();
                    trendArea.setDateRange("From " + startDate + " to " + endDate + ":");
                    for (WordDTO word : words) {
                        HasWord hasWord = trendArea.getNextWord();
                        hasWord.getText().setText(word.getDisplayString());
                        registerForClicks(hasWord, word);
                    }
                }
            }
        });
    }

    private void registerForClicks(HasWord hasWord, final WordDTO word) {
        HasClickHandlers clickHandlers = hasWord.getClickHandlers();
        HandlerRegistration addClickHandler = clickHandlers.addClickHandler(new ClickHandler() {
            
            @Override
            public void onClick(ClickEvent event) {
                placeService.changePlace(new WordDetailPlace(word));
            }
        });
        cleanup.add(addClickHandler);
    }

    
    @Override
    public void onPlaceChangeEvent(PlaceChangeEvent pce) {
        if (pce.getPlace() instanceof WordDetailPlace) {
            cleanupRegistration();
            WordDetailPlace wdp = (WordDetailPlace) pce.getPlace();
            int id = wdp.getWordId();
            populateWord(id);
        }
    }


}
