package org.recursed.w.server.handlers;

import java.util.ArrayList;

import org.recursed.w.server.ActionHandler;
import org.recursed.w.server.sql.RelatedWordResult;
import org.recursed.w.server.sql.WordsDAO;
import org.recursed.w.shared.ChronoBands;
import org.recursed.w.shared.GetGroupedWordDetailAction;
import org.recursed.w.shared.GroupedWordDetail;
import org.recursed.w.shared.WordDTO;

import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
public class GroupedWordDetailHandler implements
        ActionHandler<GetGroupedWordDetailAction, GroupedWordDetail> {

    private WordsDAO dao;

    @Inject
    public GroupedWordDetailHandler(WordsDAO dao) {
        this.dao = dao;
    }

    @Override
    public GroupedWordDetail execute(GetGroupedWordDetailAction action) {
        ArrayList<WordDTO> sourceWords = action.getWords();
        GroupedWordDetail result = new GroupedWordDetail();
        long now = System.currentTimeMillis();
        long start = now - (ChronoBands.WEEK.getRangeInMillis() * 4l);
        for (WordDTO source : sourceWords) {
            RelatedWordResult rwr = dao.getRelatedWords(source.getId(), start);
            ArrayList<WordDTO> computed = rwr.compute(5);
            for (int x=0; x<computed.size(); x++) {
                if (computed.get(x) != null) {
                    result.addLinkage(source, computed.get(x));
                } else {
                    break;
                }
            }
        }
        return result;
    }

}
