package org.recursed.w.server.handlers;

import java.util.HashMap;
import java.util.Map;

import org.recursed.w.server.ActionHandler;
import org.recursed.w.server.servlet.Histogram;
import org.recursed.w.shared.GetGroupedWordDetailAction;
import org.recursed.w.shared.GetTopWordsInChronoBands;
import org.recursed.w.shared.GetWordDetailAction;
import org.recursed.w.shared.corerpc.Action;
import org.recursed.w.shared.corerpc.BatchedAction;
import org.recursed.w.shared.corerpc.Response;
import org.recursed.w.shared.headlines.GetHeadlines;
import org.recursed.w.shared.search.GetSuggestedWordsAction;

import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
public class HandlerRegistry {

    private Map<Class<? extends Action>, ActionHandler<? extends Action, ? extends Response>> handlers;
    private Histogram histogram;

    @Inject
    public HandlerRegistry(Histogram histogram,
            TopWordsInChronoBandsHandler twicbh, WordDetailHandler wdh,
            GroupedWordDetailHandler gwdh, BatchedActionHandler bah, GetHeadlinesHandler ghh,
            SuggestWordHandler swh) {
        this.handlers = new HashMap<Class<? extends Action>, ActionHandler<? extends Action, ? extends Response>>();
        handlers.put(GetTopWordsInChronoBands.class, twicbh);
        handlers.put(GetWordDetailAction.class, wdh);
        handlers.put(GetGroupedWordDetailAction.class, gwdh);
        handlers.put(BatchedAction.class, bah);
        handlers.put(GetHeadlines.class, ghh);
        handlers.put(GetSuggestedWordsAction.class, swh);
        bah.setRegistry(this);
        this.histogram = histogram;
    }

    @SuppressWarnings("unchecked")
    public <R extends Response> R execute(Action action) {
        ActionHandler handler = getActionHandler(action);
        R r = null;
        long start = System.currentTimeMillis();
        try {
            r = (R) handler.execute(action);
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        } finally {
            long end = System.currentTimeMillis();
            histogram.record(handler, start, end);
        }
        return r;
    }

    /**
     * 
     * @param <A>
     * @param a
     * @return
     */
    @SuppressWarnings("unchecked")
    public <A extends Action> ActionHandler<A, ? extends Response> getActionHandler(
            A a) {
        return (ActionHandler<A, ?>) handlers.get(a.getClass());
    }
}
