package org.recursed.w.server.handlers;

import java.util.List;

import org.recursed.w.server.ActionHandler;
import org.recursed.w.server.sql.WordsDAO;
import org.recursed.w.shared.ChronoBands;
import org.recursed.w.shared.GetTopWordsInChronoBands;
import org.recursed.w.shared.TopWordsInChronoBands;
import org.recursed.w.shared.WordDTO;

import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
public class TopWordsInChronoBandsHandler implements
        ActionHandler<GetTopWordsInChronoBands, TopWordsInChronoBands> {

    private WordsDAO wordsDAO;
    private TopWordsInChronoBands staleResult;
    private long lastAccess = 0;
    private static final long FIVE_MINS = 1000l*60l*5l;

    @Inject
    private TopWordsInChronoBandsHandler(WordsDAO wordsDAO) {
        this.wordsDAO = wordsDAO;
    }

    @Override
    public TopWordsInChronoBands execute(GetTopWordsInChronoBands action) {
        TopWordsInChronoBands result = checkCache(action);
        if (result == null) {
            result = getFromDatabase(action);
        } 
        return result;
    }

    private TopWordsInChronoBands checkCache(final GetTopWordsInChronoBands action) {
        TopWordsInChronoBands result = staleResult;
        if (lastAccess < System.currentTimeMillis()- FIVE_MINS) {
            lastAccess = System.currentTimeMillis();
            System.out.println("STALE_CACHE in TopWordsInChronoBandsHandler");
            Runnable r = new Runnable() {
                @Override
                public void run() {
                    getFromDatabase(action);
                }
            };
            new Thread(r).start();
        }
        return result;
    }
    
    private TopWordsInChronoBands getFromDatabase(GetTopWordsInChronoBands action) {
        TopWordsInChronoBands result = new TopWordsInChronoBands();
        ChronoBands[] bands = ChronoBands.values();
        long finish = System.currentTimeMillis();
        int n = action.getNumberOfWords();
        for (ChronoBands band : bands) {
            long start = finish - band.getRangeInMillis();
            List<WordDTO> words = wordsDAO.getWords(n, start, finish);
            WordDTO[] wordArr = new WordDTO[words.size()];
            result.setWords(band, words.toArray(wordArr));
        }
        staleResult = result;
        return result;
    }
}
