package org.recursed.w.server.handlers;

import java.util.ArrayList;

import org.recursed.w.server.ActionHandler;
import org.recursed.w.server.sql.RelatedWordResult;
import org.recursed.w.server.sql.TrendCalculator;
import org.recursed.w.server.sql.WordsDAO;
import org.recursed.w.shared.ChronoBands;
import org.recursed.w.shared.GetWordDetailAction;
import org.recursed.w.shared.WordDTO;
import org.recursed.w.shared.WordDetail;
import org.recursed.w.shared.trends.TrendDTO;

import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
public class WordDetailHandler implements
        ActionHandler<GetWordDetailAction, WordDetail> {

    private WordsDAO dao;

    @Inject
    public WordDetailHandler(WordsDAO dao) {
        this.dao = dao;
    }

    @Override
    public WordDetail execute(GetWordDetailAction action) {
        int id = action.getWordId();
        WordDTO coreWord = dao.getWord(id);
        long now = System.currentTimeMillis();
        WordDetail response = new WordDetail(coreWord);
        ChronoBands band = ChronoBands.WEEK;
        long diff = 4l*(band.getRangeInMillis());
        long start = now - diff;
        RelatedWordResult rwr = dao.getRelatedWords(id, start);
        TrendCalculator tc = new TrendCalculator(rwr);
        ArrayList<TrendDTO> trends = tc.getTrends(action.getMaxWords());
        response.setTrends(trends);
        return response;
    }

}
