use DBI;
use Source;
use Word;

use strict;
use warnings;

my $connection = "DBI:mysql:database=w;host=localhost";
my $username = "w";
my $password = "dfsjklj65465!hj673";
my %properties = ('RaiseError' => 1);

# Connect to the database.

sub get_sources {
 my $dbh = DBI->connect($connection, $username, $password, \%properties);
 my $sth = $dbh->prepare("SELECT source_id, codename, rss_url FROM sources");
 my @sources = ();
 $sth->execute();
 while (my @array = $sth->fetchrow_array()) {
  my $source = new Source(@array);
  push (@sources, $source);
 }
 $sth->finish();
 $dbh->disconnect();
 return @sources;
}
 
sub is_article_analyzed {
 my $articleURL = shift;
 my $dbh = DBI->connect($connection, $username, $password, \%properties);
 my $sth = $dbh->prepare("SELECT count(*) FROM articles where url = '$articleURL'");
 my $count;
 $sth->execute();
 while (my @array = $sth->fetchrow_array()) {
  $count = $array[0];
 }
 $sth->finish();
 $dbh->disconnect();
 return $count;
}
 
sub insert_article_analysis {
 my $source = shift;
 my $title = shift;
 my $article_url = shift;
 # insert record in articles
 my $article_id = insert_article_record($source->{id}, $article_url, $title);
 my $analysis_ref = shift;
 my %analysis = %$analysis_ref;
 my %articleHash = %{$analysis{histogram}};
 my %articleWords = %{$analysis{words}};
 # print "Got an article for ", $source->{codename}, "\n  Article URL: ", $article_url, "\n";
 # print "     Article Histogram:\n";
 for my $key (keys %articleHash) {
#  print "         ",$key, "(", $articleHash{$key}, ") for: ";
  my @normal_representations = keys %{$articleWords{$key}};
  my $first_normal = $normal_representations[0];
  my $word_id = reconcile_word($key, $first_normal);
  append_histogram($article_id, $word_id, $articleHash{$key});
#  for my $word (keys %{$articleWords{$key}}) {
#   print $word, " ";
#  }
#  print "\n";
 }
}

sub insert_article_record {
 my $dbh = DBI->connect($connection, $username, $password, \%properties);
 my $sth = $dbh->prepare("insert into articles (source_id, url, title) values (?,?,?)");
 $sth->execute(shift, shift, shift);
 $sth->finish();
 my $article_id = $dbh->last_insert_id(undef, undef, "articles", undef);
 $dbh->disconnect();
 return $article_id;
} 

sub reconcile_word {
 my $stemmed = shift;
 my $normal = shift;
 my $word_id = undef;
 my $dbh = DBI->connect($connection, $username, $password, \%properties);
 my $lookup = $dbh->prepare("select word_id from words where stemmed = ?");
 $lookup->execute($stemmed);
 while (my @array = $lookup->fetchrow_array()) {
  $word_id = $array[0];
 }
 $lookup->finish();
 if (!defined $word_id) {
  my $insert = $dbh->prepare("insert into words (stemmed, normal) values (?,?)");
  $insert->execute($stemmed, $normal); 
  $insert->finish();
  $word_id = $dbh->last_insert_id(undef, undef, "words", undef);
 }
 $dbh->disconnect();
 return $word_id;
}

sub append_histogram {
 my $article_id = shift;
 my $word_id = shift;
 my $frequency = shift;
 my $dbh = DBI->connect($connection, $username, $password, \%properties);
 my $insert = $dbh->prepare("insert into word_histogram (article_id, word_id, frequency) values (?,?,?)");
 $insert->execute($article_id, $word_id, $frequency);
 $insert->finish();
 $dbh->disconnect();
}

sub get_all_words {
 my $dbh = DBI->connect($connection, $username, $password, \%properties);
 my $sth = $dbh->prepare("select w.word_id, w.normal, sum(wh.frequency) as sf from words w, word_histogram wh where wh.word_id = w.word_id group by wh.word_id order by sf asc");
 my @words = ();
 $sth->execute();
 while (my @array = $sth->fetchrow_array()) {
  my $w = new Word(@array);
  push (@words, $w);
 }
 $sth->finish();
 $dbh->disconnect();
 return @words;
}

sub mark_as_ignorable {
 my $dbh = DBI->connect($connection, $username, $password, \%properties);
 my $sth = $dbh->prepare("update words w set w.ignore = 1 where w.word_id = ?");
 $sth->execute(shift); 
 $sth->finish();
 $dbh->disconnect();
}

sub get_non_summarized_dates {
 my $dbh = DBI->connect($connection, $username, $password, \%properties);
 my $sth = $dbh->prepare("select 
      date(a.analyzed_on) an_dates
      from 
      articles a
      where 
      date(a.analyzed_on) not in
      (
      select date(sd.summarized_on) sd_found from summary_days sd
      group by sd_found
      order by sd_found asc
      )
      and date(a.analyzed_on) != date(now())
      group by an_dates
      order by an_dates asc
      ");
 my @result;
 $sth->execute();
 while (my @array = $sth->fetchrow_array()) {
  push(@result, $array[0]);
 }
 $sth->finish();
 $dbh->disconnect();
 return @result;
}

sub etl_summary_day {
 my $date_to_summarize = shift or die "no date supplied";
 my $dbh = DBI->connect($connection, $username, $password, \%properties);
 my $sth = $dbh->prepare("insert into summary_days (summarized_on, word_id, frequency) 
         select 
         date(a.analyzed_on), wh.word_id, sum(wh.frequency)
         from
         word_histogram wh,
         articles a,
         words w
         where 
         wh.article_id = a.article_id
         and date(a.analyzed_on) = ?
         and w.word_id = wh.word_id
         and w.ignore = 0
         group by wh.word_id
         order by 2, wh.word_id");
 $sth->execute($date_to_summarize); 
 $sth->finish();
 $dbh->disconnect();
}

1;