package org.recursed.w.server.servlet;

import javax.servlet.http.HttpServletRequest;

import org.recursed.w.server.handlers.HandlerRegistry;
import org.recursed.w.shared.corerpc.Action;
import org.recursed.w.shared.corerpc.ActionService;
import org.recursed.w.shared.corerpc.Response;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;
import com.google.gwt.user.server.rpc.SerializationPolicy;
import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
@SuppressWarnings("serial")
public class ActionHandlerServlet extends RemoteServiceServlet implements
        ActionService {

    private HandlerRegistry registry;

    @Inject
    private ActionHandlerServlet(HandlerRegistry registry) {
        this.registry = registry;
    }

    @SuppressWarnings("unchecked")
    @Override
    public <R extends Response> R execute(Action action) {
        return (R) registry.execute(action);
    }


    @Override
    protected SerializationPolicy doGetSerializationPolicy(
            HttpServletRequest request, String moduleBaseURL, String strongName) {
        //get the base url from the header instead of the body this way 
        //apache reverse proxy with rewrite on the header can work
        String moduleBaseURLHdr = request.getHeader("X-GWT-Module-Base");
        //System.out.println("Resolved header to: " + moduleBaseURLHdr);
        if(moduleBaseURLHdr != null){
            moduleBaseURL = moduleBaseURLHdr;
        }
        if (moduleBaseURL.indexOf("/w/w") < 0) {
            moduleBaseURL = moduleBaseURL+"w/";
        }
        //System.out.println("Wrapped up with: " + moduleBaseURL + " " + strongName);
        return super.doGetSerializationPolicy(request, moduleBaseURL, strongName);
    }

}
