package org.recursed.w.server.servlet;

import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.util.HashMap;
import java.util.Set;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.recursed.w.server.sql.Pool;

import com.google.inject.Inject;
import com.google.inject.Singleton;

@Singleton
@SuppressWarnings("serial")
public class StatsServlet extends HttpServlet {

    private Runtime runtime;
    private Pool pool;
    private Histogram histogram;

    @Inject
    public StatsServlet(Pool pool, Histogram h) {
        this.pool = pool;
        this.histogram = h;
    }
    
    @Override
    protected void service(HttpServletRequest request,
            HttpServletResponse response) throws ServletException, IOException {
        response.setContentType("text/plain");
        StringBuilder sb = new StringBuilder();
        appendServerStats(sb);
        appendDBStats(sb);
        appendHistogram(sb);
        response.getWriter().print(sb.toString());
    }

    private void appendServerStats(StringBuilder sb) {
        if (runtime == null) {
            synchronized (this) {
                runtime = Runtime.getRuntime();
            }
        }
        Date d = new Date();
        sb.append(d).append('\n').append(runtime.freeMemory()).append('/').append(runtime.totalMemory()).append('\n');
    }


    private void appendDBStats(StringBuilder sb) {
        Connection con = pool.getConnection();
        PreparedStatement st = null;
        ResultSet rs = null;
        try {
            st = con.prepareStatement("select count(*) from words");
            rs = st.executeQuery();
            if (rs.next()) {
                sb.append("Word count: ").append(rs.getInt(1)).append("\n");
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
            Pool.close(con, st, rs);
        }
    }

    private void appendHistogram(StringBuilder sb) {
        Set<Class<?>> classes = histogram.getAllKeys();
        sb.append("Histogram Data:\n");
        for (Class<?> c : classes) {
            sb.append(" ").append(c.getName()).append("\n");
            HashMap<Integer, Integer> data = histogram.getData(c);
            for (Integer bucket : data.keySet()) {
                Integer tally = data.get(bucket);
                sb.append("  ").append(bucket).append(":").append(tally).append("\n");
            }
        }
    }


}
