package org.recursed.w.server.sql;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.TreeSet;

import org.recursed.w.shared.ChronoBands;
import org.recursed.w.shared.WordDTO;

public class RelatedWordResult {

    private static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyyy-MM-dd");
    private HashMap<Integer, Integer> wordFreqSum = new HashMap<Integer, Integer>();
    private HashMap<Integer, HashSet<Integer>> wordsInTally= new HashMap<Integer, HashSet<Integer>>();
    private HashMap<Integer, ArrayList<Tuple>> wordData = new HashMap<Integer, ArrayList<Tuple>>();
    private HashMap<Integer, WordDTO> wordDTOs = new HashMap<Integer, WordDTO>();
    private Date start = new Date();

    public RelatedWordResult(Date start) {
        this.start = start;
    }

    public void addWord(WordDTO word, Date d) {
        int wordId = word.getId();
        Integer freqSumI = wordFreqSum.get(wordId);
        int freqSum = 0;
        if (freqSumI != null) {
            freqSum = freqSumI;
        }
        HashSet<Integer> currentTallyGroup = wordsInTally.get(freqSum);
        if (currentTallyGroup != null) {
            currentTallyGroup.remove(wordId);
        }
        freqSum += word.getCount();
        HashSet<Integer> newTallyGroup = wordsInTally.get(freqSum);
        if (newTallyGroup == null) {
            newTallyGroup = new HashSet<Integer>();
        }
        newTallyGroup.add(wordId);
        wordsInTally.put(freqSum, newTallyGroup);
        wordFreqSum.put(wordId, freqSum);
        ArrayList<Tuple> currentWordData = wordData.get(wordId);
        if (currentWordData == null) {
            currentWordData = new ArrayList<Tuple>();
        }
        Tuple t = new Tuple(d, word.getCount());
        currentWordData.add(t);
        wordData.put(wordId, currentWordData);
        wordDTOs.put(wordId, word);
    }

    
    private class Tuple {
        String date;
        int count;
        Tuple(Date d, int count) {
            this.date = DATE_FORMAT.format(d);
            this.count = count;
        }
    }

    public ArrayList<WordDTO> compute(int maxWords) {
        TreeSet<Integer> rankedTallys = new TreeSet<Integer>(wordsInTally.keySet());
        Iterator<Integer> it = rankedTallys.descendingIterator();
        ArrayList<WordDTO> result = new ArrayList<WordDTO>();
        int counter = 0;
        while (it.hasNext() && counter < maxWords) {
            Integer tally = it.next();
            HashSet<Integer> wordIDs = wordsInTally.get(tally);
            Iterator<Integer> wordIDIterator = wordIDs.iterator();
            while (wordIDIterator.hasNext() && counter < maxWords) {
                int wordId = wordIDIterator.next();
                WordDTO dto = wordDTOs.get(wordId);
                result.add(dto);
                counter++;
            }
        }
        return result;
    }

    public int[] getValues(WordDTO w) {
        int id = w.getId();
        ArrayList<Tuple> tuples = wordData.get(id);
        int[] result = new int[28];
        int pointer = 0;
        long curDate = start.getTime();
        for (int x=0; x<result.length; x++) {
            String curDateString = DATE_FORMAT.format(curDate);
            if (pointer == tuples.size()) {
                // no more data to read..
                break;
            }
            if (curDateString.equals(tuples.get(pointer).date)) {
                result[x] = tuples.get(pointer).count;
                pointer++;
            }
            curDate += ChronoBands.DAY.getRangeInMillis();
        }
        return result;
    }

    public String[] getDates() {
        String[] result = new String[28];
        long now = System.currentTimeMillis();
        long curDate = start.getTime();
        for (int x=0; now > curDate && x<28 ; x++) {
            result[x] = DATE_FORMAT.format(curDate);
            curDate += ChronoBands.DAY.getRangeInMillis();
        }
        return result;
    }

}
