package org.recursed.w.server.sql;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.TreeSet;

import org.recursed.w.shared.WordDTO;
import org.recursed.w.shared.trends.TrendDTO;

public class TrendCalculator {

    private RelatedWordResult relatedWordResult;
    private HashMap<Signature, ArrayList<WordDTO>> signatureToWordMap;
    private String[] dates;

    public TrendCalculator(RelatedWordResult relatedWordResult) {
        this.relatedWordResult = relatedWordResult;
        signatureToWordMap = new HashMap<Signature, ArrayList<WordDTO>>();
    }

    private void calculateTrends(int numberOfWords) {
        ArrayList<WordDTO> words = relatedWordResult.compute(numberOfWords);
        this.dates = relatedWordResult.getDates();
        for (WordDTO word : words) {
            int[] values = relatedWordResult.getValues(word);
            Signature sig = resolveSignature(values);
            //System.out.println(word.getDisplayString() + " sig: " + Arrays.toString(sig));
            ArrayList<WordDTO> wordsForSignature = signatureToWordMap.get(sig);
            if (wordsForSignature == null) {
                wordsForSignature = new ArrayList<WordDTO>();
            }
            wordsForSignature.add(word);
            sig.incrementWeight(values);
            signatureToWordMap.put(sig, wordsForSignature);
        }
    }

    public ArrayList<TrendDTO> getTrends(int maxNumber) {
        calculateTrends(20);
        TreeSet<Signature> sortedKeys = new TreeSet<Signature>(signatureToWordMap.keySet());
        ArrayList<TrendDTO> result = new ArrayList<TrendDTO>();
        int x= 0;
        for (Signature s: sortedKeys) {
            if (x == maxNumber) {
                break;
            }
            TrendDTO t = new TrendDTO();
            ArrayList<WordDTO> words = signatureToWordMap.get(s);
            t.addWords(words);
            ArrayList<String> ds = s.getMatchingDates();
            t.addDates(ds);
            result.add(t);
        }
        return result;
    }
    
    private Signature resolveSignature(int[] wordValuesInTime) {
        int[] result = new int[wordValuesInTime.length];
        for (int x=0; x<result.length; x++) {
            if (wordValuesInTime[x] > 0) {
                result[x] = 1;
            }
        }
        return new Signature(result);
    }

    class Signature implements Comparable<Signature> {
        int[] values;
        int weight;

        public Signature(int[] values) {
            this.values = values;
        }

        public void incrementWeight(int[] trueValues) {
            for (int v : trueValues) {
                if (v > 1) {
                    v = 2;
                }
                weight += v;
            }
        }

        public ArrayList<String> getMatchingDates() {
            ArrayList<String> result = new ArrayList<String>();
            for (int x=0; x<values.length; x++) {
                if (values[x] != 0) {
                    result.add(dates[x]);
                }
            }
            return result;
        }

        @Override
        public int hashCode() {
            long result = 0;
            long powTen = 1;
            for (int x = 0; x<values.length; x++) {
                result = result + (powTen * values[x]);
                powTen = powTen * 10;
            }
            return new Long(result).hashCode();
        }

        @Override
        public boolean equals(Object obj) {
            if (obj instanceof Signature) {
                Signature other = (Signature) obj;
                if (this.hashCode() == other.hashCode()){
                    return true;
                } else {
                    return false;
                }
            } else {
                return false;
            }
        }

        @Override
        public int compareTo(Signature o) {
            return this.weight - o.weight;
        }

    }
}
