use XML::FeedPP;
use HTML::TreeBuilder;
use HTML::FormatText;
use Lingua::Sentence;

use PorterStemmer;
use WordNetSupport;
use LinkParserSupport;

use strict;
use warnings;

sub extractText {
 my $chunk = shift;
 my $tree  = HTML::TreeBuilder->new_from_content($chunk);
 #$tree->parse($chunk);
 my $formatter =
   HTML::FormatText->new( leftmargin => 0, rightmargin => 50000000 );
 my $result = $formatter->format($tree);
 $tree->delete();
 # remove extranneous [IMAGE] and such tags
 #print "Formatted: \n$result\n";
 $result =~ s/\[.*\]//g;    # this removes the [IMAGE] and other tags
 $result =~ s/[ \t]+\*.+\n/ /g;  # eliminate the trailing bullets in foxnews
 $result =~ s/Email this Article.+//g;
 $result =~ s/\n/ /g;  # eliminate newlines and make them delimited sentences.
  return $result;
}

sub getItems {
 my $source = shift;
 my $feed   = XML::FeedPP->new($source);
 return $feed->get_item();
}

my $splitter = Lingua::Sentence->new("en");

sub analyze {
 my $content = shift;
 my %articleHash = ();
 my %articleWords = ();
 #print "\n Going to try and parse: \n $content\n";
 my @sentences = split( /\n/, $splitter->split($content) );
 my $hadErrs = 0; 
 foreach my $sentence (@sentences) {
   print "..........\n";
  my %wordsOfInterest = get_words_of_interest($sentence);  
  if (scalar(%wordsOfInterest) eq "0") {
   print "!";
   $hadErrs = 1;
   # we failed to run the linker..
   # manually add words via WordNet..
   my @words = split(/\s/, $sentence);
   my $numWords = @words;
   for my $examine (@words) {
    if (lookupWord($examine) == 1) {
     $wordsOfInterest{$examine} = $examine;
    }
   }
  } else {
   print ".";
  }
  # At this point, we should have the sentence hash,
  # from the Linker, or from WordNet.
  # We now iterate through each word returned, 
  # and add it to the a hash articleWords that maintains
  # a histogram of word frequencies.
  # Additionally, we hold store references of the word in
  # articleWords.
  for my $key (keys(%wordsOfInterest)) {
   my $word = stem(lc($key));
   $articleWords{$word}{$key}="a";
   my $currentValue = $articleHash{$word};
   if (defined($currentValue)) {
    $currentValue++;
   } else {
    $currentValue = 1;
   }
   $articleHash{$word} = $currentValue;
  }
 }
 print "\n";
 my %analysis = ();
 $analysis{histogram}=\%articleHash;
 $analysis{words}=\%articleWords;

 if ($hadErrs == 1) {
  print "PARSE_FAIL_IN: \n$content\n";
 }
 return \%analysis;
}

my $content = "Italian Premier Silvio Berlusconi is vowing not to quit, as long as he's needed. And he predicts he'll be around for a long time, saying he'll live to 120. Berlusconi had a pacemaker installed in 2006 after he collapsed during an appearance and has said he has beaten prostate cancer.";
$content = "The negotiations on weapon management are failing. This is concerning, as failure to keep the parties at the negotiating table will result in illicit weapons on the black market";
$content = "A punishing storm system that began Thursday evening continued to pummel parts of the Deep South into the early hours of Saturday morning. Tornadoes and vicious winds left 17 people dead across four states, officials said. The storm killed seven people in Alabama overnight Friday, according to the governor’s office. All were in their homes at the time. The Alabama Emergency Management Agency said that more than a dozen counties throughout the state were reporting damage. In southwest Alabama, a mother and her two adolescent children were killed when their double-wide mobile home was lifted off its foundation and thrown about 100 yards, landing upside down. The structure was crushed. \“Nothing on the mobile home was over knee high,\” said Terry Beasley, the chief deputy sheriff of Washington County.";

my $analysis_ref = analyze($content);
my %analysis = %$analysis_ref;
my %articleHash = %{$analysis{histogram}};
my %articleWords = %{$analysis{words}};
print "\nArticle Histogram:\n";
for my $key (keys %articleHash) {
 print $key, "(", $articleHash{$key}, ") for: ";
 for my $word (keys %{$articleWords{$key}}) {
  print $word, " ";
 }
 print "\n";
}

print stem("Libya");
print stem("Libyan");

1;
