use warnings;
use strict;

use Lingua::LinkParser;

sub get_words_of_interest {
 my $t1                    = time;
 my %wordsOfInterest       = ();
 my @nonParseableSentences = ();
 my $sent                  = shift;

 # this is a very simple definition that I see in use in the examples
 # I would like to move to just this one definition instead of the numerous
 # types with various panic modes and such..
 my $parser                = new Lingua::LinkParser(verbosity => 0, display_on => 0);
 $parser->opts(
   'max_null_count' => 3,
   'min_null_count' => 1
 );
 my $sentence = $parser->create_sentence($sent);
 if ( $sentence->num_linkages == 0 ) {
  $parser->opts(
   'min_null_count' => 1,
   'max_null_count' => $sentence->length
  );
  $sentence = $parser->create_sentence($sent);
  if ( $sentence->num_linkages == 0 ) {
   $parser->opts(
    'disjunct_cost'        => 3,
    'min_null_count'       => 1,
    'max_null_count'       => 30,
    'max_parse_time'       => 60,
    'islands_ok'           => 1,
    'short_length'         => 6,
    'all_short_connectors' => 1,
    'linkage_limit'        => 100
   );
   $sentence = $parser->create_sentence($sent);
  }
 }


 my @bigStruct = $sentence->get_bigstruct;


 my $wordCount = @bigStruct;
 foreach my $i ( 1 .. $wordCount ) {
  my $word = $bigStruct[$i]->{word};

  # April 10th: This is a much cleaner looking way to pull words.
  # I think the code after this while loop needs to be refactored to be placed
  # in this loop, and use the links a lot more. Also, note that the first pass
  # parser is a lot simpler than prior. This may have significant speedup over
  # the prior implementation with all sorts of switches to the config.
  while (my ($k,$v) = each %{$bigStruct[$i]->{links}} ) { 
    # Linkages of type G are typically nouns like names
    # We want to pull them in only one directional sequence, hence the extra check
    if ($k =~ m/G.*/ && $i<$v) {
      my $concatWord = $sentence->get_word($i)." ".$sentence->get_word($v);
      my $rawWord = $word." ".$bigStruct[$v]->{word};
      $wordsOfInterest{$concatWord}=$rawWord;
      print "Found a concat: ".$concatWord, "\n";
    }
  }

 #while ((my $linkageType, my $wordID) = each %{$bigStruct[$i]->{links}}){
 # print $word, " is linked by ", $linkageType, " to ", $bigStruct[$wordID]->{word}, "\n";
 #}

  
  if ( defined($word) && $word =~ m/.*\.[a,b,g,m,n,l,f,g].*/ ) {
   print $word, "\n";
   
   if ($word =~ m/.*\.[f].*/) {
    #TODO: We want to expand this usage to test for proper nouns using the G connector recursively.
    # Eg:
    #    +----G---+----G----+---AN---+---Ss--+-Pp-+
    #    |        |         |        |       |    |
    # George.b Herbert.m Walker.m Bush[?].n is.v here 
    #
    # this is a noun.. testing for AN linkages
    while ((my $linkageType, my $wordID) = each %{$bigStruct[$i]->{links}}){
     if ($i == $wordID - 1) {
      my $compositeWord = $sentence->get_word($i). " ". $sentence->get_word($wordID);
      #print $sentence->get_word($i), " ", $sentence->get_word($wordID), "\n";
      $wordsOfInterest{$compositeWord} = $word;
     }
     #print $word, " is linked by ", $linkageType, " to ", $bigStruct[$wordID]->{word}, "\n";
    }
   }
   my $wordOfInterest = $sentence->get_word($i);
   $wordsOfInterest{$wordOfInterest} = $word;
  }
 }

 my $t4 = time;

 #print "Big struct: ", ($t4 - $t3), "s\n";

 return %wordsOfInterest;
}

#my $content = "The Obama administration is monitoring and planning its next move, with embassy personnel still in harm's way and attempts to reach Gbagbo unsuccessful.";
#my $content = "That morning newspapers carried the story of John Galliano’s dismissal from Dior.";
#my %map = get_words_of_interest($content);
#foreach my $key (keys %map) {
# print $key, ", ", $map{$key}, "\n";
#}

1;
